@@ -26,6 +26,8 @@
 
 static ScanDevice *default_device = NULL;
 
+static gboolean have_devices = FALSE;
+
 static GUdevClient *udev_client;
 
 static SimpleScan *ui;
@@ -62,6 +64,7 @@ update_scan_devices_cb (Scanner *scanner, GList *devices)
             devices_copy = g_list_prepend (devices_copy, default_device);
     }
 
+    have_devices = devices_copy != NULL;
     ui_set_scan_devices (ui, devices_copy);
   
     g_list_free (devices_copy);
@@ -83,7 +86,7 @@ static Page *
 append_page ()
 {
     Page *page;
-    Orientation orientation = TOP_TO_BOTTOM;
+    ScanDirection scan_direction = TOP_TO_BOTTOM;
     gboolean do_crop = FALSE;
     gchar *named_crop = NULL;
     gint width = 100, height = 100, dpi = 100, cx, cy, cw, ch;
@@ -98,7 +101,7 @@ append_page ()
   
     /* Copy info from previous page */
     if (page) {
-        orientation = page_get_orientation (page);
+        scan_direction = page_get_scan_direction (page);
         width = page_get_width (page);
         height = page_get_height (page);
         dpi = page_get_dpi (page);
@@ -110,7 +113,7 @@ append_page ()
         }
     }
 
-    page = book_append_page (book, width, height, dpi, orientation);
+    page = book_append_page (book, width, height, dpi, scan_direction);
     if (do_crop) {
         if (named_crop)  {
             page_set_named_crop (page, named_crop);
@@ -202,7 +205,7 @@ scanner_page_info_cb (Scanner *scanner, ScanPageInfo *info)
 
     /* Add a new page */
     page = append_page ();
-    page_set_scan_area (page, info->width, info->height, info->dpi);
+    page_set_page_info (page, info);
 
     /* Get ICC color profile */
     /* FIXME: The ICC profile could change */
@@ -261,7 +264,7 @@ scanner_failed_cb (Scanner *scanner, GError *error)
                        /* Title of error dialog when scan failed */
                        _("Failed to scan"),
                        error->message,
-                       TRUE);
+                       have_devices);
     }
 }
 
@@ -305,54 +308,6 @@ cancel_cb (SimpleScan *ui)
 }
 
 
-static gboolean
-save_book_by_extension (GFile *file, GError **error)
-{
-    gboolean result;
-    gchar *uri, *uri_lower;
-
-    uri = g_file_get_uri (file);
-    uri_lower = g_utf8_strdown (uri, -1);
-    if (g_str_has_suffix (uri_lower, ".pdf"))
-        result = book_save (book, "pdf", file, error);
-    else if (g_str_has_suffix (uri_lower, ".ps"))
-        result = book_save (book, "ps", file, error);
-    else if (g_str_has_suffix (uri_lower, ".png"))
-        result = book_save (book, "png", file, error);
-    else if (g_str_has_suffix (uri_lower, ".tif") || g_str_has_suffix (uri_lower, ".tiff"))
-        result = book_save (book, "tiff", file, error);
-    else
-        result = book_save (book, "jpeg", file, error);
-
-    g_free (uri);
-    g_free (uri_lower);
-
-    return result;
-}
-
-
-static void
-save_cb (SimpleScan *ui, const gchar *uri)
-{
-    GError *error = NULL;
-    GFile *file;
-
-    g_debug ("Saving to '%s'", uri);
-
-    file = g_file_new_for_uri (uri);
-    if (!save_book_by_extension (file, &error)) {
-        g_warning ("Error saving file: %s", error->message);
-        ui_show_error (ui,
-                       /* Title of error dialog when save failed */
-                       _("Failed to save file"),
-                       error->message,
-               FALSE);
-        g_error_free (error);
-    }
-    g_object_unref (file);
-}
-
-
 static gchar *
 get_temporary_filename (const gchar *prefix, const gchar *extension)
 {
@@ -625,7 +580,6 @@ main (int argc, char **argv)
     book = ui_get_book (ui);
     g_signal_connect (ui, "start-scan", G_CALLBACK (scan_cb), NULL);
     g_signal_connect (ui, "stop-scan", G_CALLBACK (cancel_cb), NULL);
-    g_signal_connect (ui, "save", G_CALLBACK (save_cb), NULL);
     g_signal_connect (ui, "email", G_CALLBACK (email_cb), NULL);
     g_signal_connect (ui, "quit", G_CALLBACK (quit_cb), NULL);
 
